#include "define.h"
#include "A5133reg.h"
#include "A5133Config.h"
#include <bcm2835.h>
#include <stdint.h>  // 包含标准的整数类型定义
#include <stdio.h>

uint8_t Flag_MASTER = 0;  // 1=TX; 0=RX
uint16_t RxCnt, Err_Loss, Err_Frame;
uint32_t Err_BitCnt;
uint8_t tmpbuf[64];

const Uint8 BitCount_Tab[16] = {0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4};
const Uint8 ID_Tab[8]={0xC3,0x3C,0x95,0x6A,0x36,0x75,0xC5,0x2A};//ID code
const Uint8 KeyData_Tab[16]={0x00,0x00,0x00,0x00,0x00,0x0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}; //keyData code
const Uint8 FCB_Tab[20]={0x00,0x00,0x00,0x00,0x00,0x15,0x20,0x25,0x30,0x35,0x40,0x45,0x50,0x55,0x60,0x65,0x70,0x75,0x80,0x85}; //keyData code
const Uint8 PN9_Tab[]=
{   0xFF,0x83,0xDF,0x17,0x32,0x09,0x4E,0xD1,
    0xE7,0xCD,0x8A,0x91,0xC6,0xD5,0xC4,0xC4,
    0x40,0x21,0x18,0x4E,0x55,0x86,0xF4,0xDC,
    0x8A,0x15,0xA7,0xEC,0x92,0xDF,0x93,0x53,
    0x30,0x18,0xCA,0x34,0xBF,0xA2,0xC7,0x59,
    0x67,0x8F,0xBA,0x0D,0x6D,0xD8,0x2D,0x7D,
    0x54,0x0A,0x57,0x97,0x70,0x39,0xD2,0x7A,
    0xEA,0x24,0x33,0x85,0xED,0x9A,0x1D,0xE0,
};
#define GIO2 RPI_V2_GPIO_P1_07  // Define the GPIO 4 pin number (BCM2835 numbering)
typedef unsigned char Uint8;
Uint8 RF_Init(void);
void Err_State(void);
void RF_Reset(void);
void RF_WriteReg(Uint8, Uint8);
void RF_WriteID(const uint8_t *ptr);
void RF_ReadID(Uint8* ptr);
void RF_Config(void);
void RF_WritePage(Uint8 addr, Uint8 wbyte, Uint8 page);
void RF_TrimmedValue_Init(void);
void RF_StrobeCmd(uint8_t cmd);
Uint8 RF_Cal(void);
Uint8 RF_ReadReg(Uint8);
Uint8 RF_Cal_CHGroup(Uint8 ch);
void RF_FIFOLength(Uint16 len);
void RF_SetCH(Uint8);
void RF_FIFOWrite(Uint8);
Uint8 RF_LVR_Check(void);
void RxPacket(void);
//===========================================================
int main(void)
{
    if (!bcm2835_init()) {
        printf("BCM2835 初始化失敗！\n");
        return 1;
    }
    // 设置 GIO2 为输入模式
    bcm2835_gpio_fsel(GIO2, BCM2835_GPIO_FSEL_INPT);
    // 设置 GIO2 的下拉电阻
    bcm2835_gpio_set_pud(GIO2, BCM2835_GPIO_PUD_DOWN);

    if (!bcm2835_spi_begin())
    {
      printf("bcm2835_SPI_啟動失敗\n");
      return 1;
    }
    // 设置 SPI 位序为 MSB First
    bcm2835_spi_setBitOrder(BCM2835_SPI_BIT_ORDER_MSBFIRST);

    // 设置 SPI 模式为 Mode 0 (CPOL = 0, CPHA = 0)
    bcm2835_spi_setDataMode(BCM2835_SPI_MODE0);

    // 设置 SPI 时钟分频器（时钟速度）
    bcm2835_spi_setClockDivider(BCM2835_SPI_CLOCK_DIVIDER_256);  // 设置时钟为 1 MHz

    // 选择 SPI 通道的片选信号（CS0）
    bcm2835_spi_chipSelect(BCM2835_SPI_CS0);

    // 设置片选信号的极性（低电平有效）
    bcm2835_spi_setChipSelectPolarity(BCM2835_SPI_CS0, LOW);
    bcm2835_delay(100);//Delay100ms
//Only Tx 
if(Flag_MASTER)
   {
	while(1)
	{
	 if(RF_Init())
	   Err_State();
	 printf("TX_RF Init Finish\n");
	 RF_WriteReg(GIO1_REG,0x19);//gio1 pin=SDO
	 RF_WriteReg(GIO2_REG,0x01);//gio2 pin=WTR
	 RF_WriteReg(CKO_REG,0x02); //cko pin=dck
         printf("TX_GIO1,2&CKO Setup Finish\n");
	 RF_FIFOLength(64-1);//64 bytes
	 RF_StrobeCmd(CMD_PLL);
	 RF_SetCH(80);//freq=5805.001MHz  80
	 printf("TX_FIFO Len & channel Setup Finish\n");
	 /*
	  To set RF channel
	  RF base Freq : 5725.001MHz
	  Channel step : 1MHz
	  SetCH        : 80
	  RF Frequency = RF base Freq + (Channel step * SetCH) = 5725.001MHz + ( 1MHz * 80 ) = 5805.001MHz 
	*/
		while(1)
		{
		  RF_FIFOWrite(64); //write data to tx fifo
	          printf("write data to TXBuf Finish\n");
		  if(RF_LVR_Check())/* check if BOD reset has occurred */
		     break;/* exit loop & re-RF init */
		  else
		   {
		     RF_StrobeCmd(CMD_TX);/* transmit packet */
		     printf("TX Data Sending\n");
		   }
		   bcm2835_delayMicroseconds(10);
		   while(bcm2835_gpio_lev(GIO2));  // 只要 GIO2 为高电平就继续等待
		   printf("Finish TX 1 Data packet\n");
                }
	}

   }
//Only RX
else
 {
	 while(1)
	 {
	      if(RF_Init())//init RF
		Err_State();
	      printf("RX_RF Init Finish\n");

	      RF_WriteReg(GIO1_REG,0x19);//gio1 pin=SDO
	      RF_WriteReg(GIO2_REG,0x01);//gio2 pin=WTR
	      RF_WriteReg(CKO_REG,0x02); //cko pin=dck
	      printf("RX_GIO1,2&CKO Setup Finish\n");

              RF_FIFOLength(64-1);//64 bytes  
	      RF_StrobeCmd(CMD_PLL);
	      RF_SetCH(80);//freq=5805.001MHz
	      printf("RX_FIFO Len & channel Setup Finish\n");

	      RxCnt = 0;
	      Err_Loss=0;
	      Err_Frame=0;
	      Err_BitCnt = 0;
              while(1)
		{
		 if(RF_LVR_Check())//check if BOD reset has occurred
		   break;//exit loop & re-RF init
		 else
		 {
		   RF_StrobeCmd(CMD_RX);
		   printf("Receiving Rx Data\n");
		 }
		 bcm2835_delayMicroseconds(10);  // 10 microseconds delay
		 while(bcm2835_gpio_lev(GIO2));  // 只要 GIO2 为高电平就继续等待
		 RxPacket();
		 printf("Finish RX %u Data Packet\n",RxCnt);
		}

	 }
 }

}
Uint8 RF_Init(void)
{
	Uint8 i;
	Uint8 id[8] = {0};
	RF_Reset();/* reset RF chip */
	RF_WriteID(ID_Tab);/* write ID code */
	RF_WriteReg(GIO1_REG, 0x19);
        RF_ReadID(id);
	//check ID
	for (i=0; i<8; i++)
	{
		if (id[i] ^ ID_Tab[i])
		{
		 printf("ID Check Fail\n");
		 return 1;
		}
	}
	//load config
	RF_Config();
	RF_TrimmedValue_Init();/* load trimming value */
	if(RF_Cal())/* rf calibration */
	  return 1;
	return 0;
}
void Err_State(void)
{
	//ERR display
	//Error Proc...
	//...
	while(1);
}
void RF_Reset(void)
{
	RF_WriteReg(MODE_REG, 0x00); //reset RF chip
}
void RF_WriteReg(Uint8 addr, Uint8 dataByte)
{
	 uint8_t data[2];
	 data[0]=addr;
	 data[1]=dataByte;
	 printf("Sending data over SPI:Reg=0x%02X,Vaule=0x%02X,", data[0], data[1]);
 	 bcm2835_spi_writenb((char*)data, 2);  // 連續寫2Byte
	 printf("Send Finish\n");
}
void RF_WriteID(const uint8_t *ptr)
{
	Uint8 i;
	uint8_t data[9];//1byte adderss & 8 byte ID Code
	data[0]=IDCODE_REG;//address
	for (i=0; i < 8; i++)
	{
	 data[i+1]= *ptr;
	 ptr++;
	}
        printf("Sending data over SPI:Reg=0x%02X,Vaule=0x%02X,0x%02X,0x%02X,0x%02X,0x%02X,0x%02X,0x%02X,0x%02X", data[0],data[1],data[2],data[3],data[4],data[5],data[6], data[7],data[8]);
	bcm2835_spi_writenb((char*)data, 9);  // 连续写入 9 个字节
	printf(",ID Send Finish\n");
}
void RF_ReadID(Uint8* ptr)
{
	uint8_t buffer[9];
	buffer[0]=(IDCODE_REG | 0x40);
	// 其余字节初始化为 0，用于接收数据
    	for (int i = 1; i < 9; i++) {
        	buffer[i] = 0x00;
    	}
	// 在同一个片选信号周期内发送 1 字节并接收 8 字节
        bcm2835_spi_transfern((char*)buffer, 9);  // 连续写 1 字节，读 8 字节
        printf("Read ID Code=");
	for (int i=1; i<9; i++)
	{
          *ptr++ = buffer[i];
	   printf("0x%02X ", buffer[i]);
	}
	printf("\n");
}
void RF_Config(void)
{
	Uint8 i;
	printf("Load Config Start\n");
        for (i=0x01; i<=0x04; i++)
                RF_WriteReg(i, A5133_RFConfigTab_Main[i]);
	for (i=0x07; i<=0x1F; i++)
		RF_WriteReg(i, A5133_RFConfigTab_Main[i]);
	for (i=0; i<=12; i++)//0x20 code1
		RF_WritePage(0x20, A5133_RFConfigTab_Addr0x20[i], i);
	for (i=0; i<=12; i++)//0x21 code2
		RF_WritePage(0x21, A5133_RFConfigTab_Addr0x21[i], i);
	for (i=0; i<=5; i++)//0x22 code3
		RF_WritePage(0x22, A5133_RFConfigTab_Addr0x22[i], i);
	for (i=0x23; i<=0x29; i++)
		RF_WriteReg(i, A5133_RFConfigTab_Main[i]);
	for (i=0; i<=12; i++)//0x2A DAS
		RF_WritePage(0x2A, A5133_RFConfigTab_Addr0x2A[i], i);
	for (i=0x2B; i<=0x35; i++)
		RF_WriteReg(i, A5133_RFConfigTab_Main[i]);
	RF_WriteReg(0x37, A5133_RFConfigTab_Main[0x37]);
	for (i=0; i<=11; i++)//0x38 ROM
		RF_WritePage(0x38, A5133_RFConfigTab_Addr0x38[i], i);
	for (i=0x39; i<=0x3C; i++)
		RF_WriteReg(i, A5133_RFConfigTab_Main[i]);
	RF_WriteReg(0x3E, A5133_RFConfigTab_Main[0x3E]);
        printf("Load Config Finish\n");

}
void RF_WritePage(Uint8 addr, Uint8 wbyte, Uint8 page)
{
	RF_WriteReg(RFANALOG_REG, (A5133_RFConfigTab_Main[0x35]&0x0F) | page<<4);
	RF_WriteReg(addr, wbyte);
}
void RF_TrimmedValue_Init(void)
{
	Uint8 i;
	Uint8 trimValue[8];
	//Uint8 tmp_checksum;

	//trimValue[0]=FBG
	//trimValue[1]=CTR
	//trimValue[2]=BDC
	//trimValue[3]=STM
	//trimValue[4]=Checksum for trimvalue[0]~trimvalue[3]
	//trimValue[5]=CSXTL
	//trimValue[6]=FBG_CP
	//trimValue[7]=Checksum for customer

	RF_WritePage(ROMP_REG, A5133_RFConfigTab_Addr0x38[9] | 0xA0, 9);//enable EFSW=1, EFRE=1

	bcm2835_delay(5);//wait for stability

	Uint8 buffer[9];  // Total 9 bytes (1 byte to send + 8 bytes to read)
	buffer[0] = USID_REG | 0x40;  // Example: 1-byte command to send
	for (i = 1; i < 9; i++)
	{
         buffer[i] = 0x00;  // Fill the rest with dummy data (zeros)
    	}
	 bcm2835_spi_transfern((char*)buffer, 9);  // 连续写 1 字节，读 8 字节
    	for (i = 1; i < 9; i++)
	{
          trimValue[i-1]=buffer[i];
    	}

	RF_WritePage(ROMP_REG, A5133_RFConfigTab_Addr0x38[9], 9);//disable EFSW=1, EFRE=1

	if((trimValue[0] + trimValue[1]) == trimValue[4]) //case1-only FT
	{
		if((trimValue[0]!=0) && (trimValue[1]!=0))
		{
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[1] & 0xE0) | trimValue[0], 1);//FBG
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[2] & 0xC0) | trimValue[1], 2);//CTR          
		}
		else
			Err_State();
   }
	else if((trimValue[0] + trimValue[1] + trimValue[2] + trimValue[3]) == trimValue[4]) //case2-CP+FT
	{
		if((trimValue[0]!=0) && (trimValue[1]!=0) && (trimValue[2]!=0) && (trimValue[3]!=0)) 
		{
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[1] & 0xE0) | trimValue[0], 1);//FBG
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[2] & 0xC0) | trimValue[1], 2);//CTR      
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[0] & 0x03) | (trimValue[2]<<2), 0);//BDC
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[4] & 0x40) | trimValue[3], 4);//STM
		}
		else
			Err_State();
	}
	else //only CP
	{
		if((trimValue[0]==0) && (trimValue[1]!=0) && (trimValue[2]!=0) && (trimValue[3]!=0) && (trimValue[4]==0) && (trimValue[6]!=0)) 
		{
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[1] & 0xE0) | trimValue[6], 1);//FBG
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[2] & 0xC0) | trimValue[1], 2);//CTR      
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[0] & 0x03) | (trimValue[2]<<2), 0);//BDC
			RF_WritePage(ROMP_REG, (A5133_RFConfigTab_Addr0x38[4] & 0x40) | trimValue[3], 4);//STM
		}
		else
			Err_State(); 
	}

}
Uint8 RF_Cal(void)
{
	Uint8 tmp;
	Uint8 rhc,rlc,fb,fbcf,fcd;

	RF_StrobeCmd(CMD_PLL); //calibration @PLL state

	RF_WriteReg(RFANALOG_REG, 0);

	//IF,RSSI,RC procedure
	RF_WriteReg(CALIBRATION_REG, 0x23);
	do{
		tmp = RF_ReadReg(CALIBRATION_REG)&0x23;
	}while(tmp);

	//calibration VBC,VDC procedure
	if(RF_Cal_CHGroup(20)) //calibrate channel group Bank I
		return 1;
	if(RF_Cal_CHGroup(60)) //calibrate channel group Bank II
		return 1;
	if(RF_Cal_CHGroup(100)) //calibrate channel group Bank III
		return 1;

	RF_StrobeCmd(CMD_STBY); //return to STBY state

	//for check
	tmp = RF_ReadReg(IFCAL1_REG);
	fb = tmp & 0x0F;
	fbcf = (tmp>>4) & 0x01;

	tmp = RF_ReadReg(IFCAL2_REG);
	fcd = tmp & 0x1F;

	rhc = RF_ReadReg(RXGAIN2_REG);
	rlc = RF_ReadReg(RXGAIN3_REG);
	Uint8 Mem_RH = rhc;
	Uint8 Mem_RL = rlc;

	if(fbcf)
		return 1;//error

	return 0;
}
void RF_StrobeCmd(uint8_t cmd)
{
   bcm2835_spi_transfer(cmd);
   printf("Send CMD:0x%02X\n",cmd);
}
Uint8 RF_ReadReg(Uint8 addr)
{
    Uint8 buffer[2];

    buffer[0] = addr | 0x40;  // Bit7 cmd=0, bit6 r/w=1 (read command)
    buffer[1] = 0x00;         // Dummy byte to read data
    printf("Read Address:0x%02X",buffer[0]);
    // Perform SPI transaction (send address, then read the response)
    bcm2835_spi_transfern((char*)buffer, 2);  // 连续写 1 字节，读 1 字节
    // The response from the SPI device is now in buffer[1]
    printf(",Value:0x%02X\n",buffer[1]);
    return buffer[1];
}
Uint8 RF_Cal_CHGroup(Uint8 ch)
{
	Uint8 tmp;
	Uint8 vb,vbcf,vcb,vccf;
	Uint8 deva,adev;

	RF_WriteReg(PLL1_REG, ch);
	RF_WriteReg(CALIBRATION_REG, 0x1C);
	do{
            tmp = RF_ReadReg(CALIBRATION_REG)&0x1C;
	}while (tmp);

	//for check
	tmp = RF_ReadReg(VCOCCAL_REG);
	vcb = tmp & 0x0F;
	vccf = (tmp>>4) & 0x01;

	tmp = RF_ReadReg(VCOCAL1_REG);
	vb = tmp & 0x0F;
	vbcf = (tmp >>4) & 0x01;

	tmp = RF_ReadReg(VCODEVCAL1_REG);
	deva = tmp;

	tmp = RF_ReadReg(VCODEVCAL2_REG);
	adev = tmp;

	if(vbcf)
		return 1;//error

	return 0;
}
void RF_FIFOLength(Uint16 len)
{
    Uint8 buffer[3];  // Buffer for the 3 bytes to send

    // Prepare the buffer for the SPI transaction
    buffer[0] = FIFO1_REG;       // First byte: register address (FIFO1_REG)
    buffer[1] = len & 0xFF;      // Second byte: low byte of len
    buffer[2] = len >> 8;        // Third byte: high byte of len
    printf("Sending data over SPI:Reg=0x%02X,Vaule=0x%02X,0x%02X,", buffer[0], buffer[1],buffer[2]);
    bcm2835_spi_writenb((char*)buffer, 3);  // 連續寫3Byte
    printf("Send Finish\n");
}
void RF_SetCH(Uint8 ch)
{
 RF_WriteReg(PLL1_REG, ch); //RF freq = RFbase + (CH_Step * ch)
}
void RF_FIFOWrite(Uint8 len)
{
	Uint8 buffer[len + 1];  // Buffer to hold FIFO register address + data
	Uint8 i;
	RF_StrobeCmd(CMD_TFR);//TX fifo pointer reset

	buffer[0] = FIFO_REG;  // First byte: FIFO register address
	for (i = 0; i < len; i++) {
        buffer[1 + i] = PN9_Tab[i];  // Subsequent bytes: Data to write to FIFO
    	}
	bcm2835_spi_writenb((char*)buffer, len+1);
}
Uint8 RF_LVR_Check(void)
{
	Uint8 tmp;

	tmp = RF_ReadReg(VCOCAL2_REG);
	if(tmp == 0xFF)//default reset value 0xFF
		return 1;
	else
		return 0;
}
void RxPacket(void) 
{
    uint8_t i, recv, tmp, err = 0;
    uint8_t buffer[65];  // Buffer to hold address and 64 bytes of data

    RxCnt++;  // Increment packet receive counter
    err = 0;  // Reset error flag

    // Step 1: Reset the RX FIFO pointer
    RF_StrobeCmd(CMD_RFR);

    // Step 2: Prepare buffer to send the address and read 64 bytes in one transaction
    buffer[0] = FIFO_REG | 0x40;  // Read command for FIFO (address byte)

    // Step 3: Perform SPI transaction (send address and read 64 bytes)
    bcm2835_spi_transfern((char*)buffer, 65);  // 连续写 1 字节，读 64 字节
   // Step 4: Process the received data
    for (i = 1; i < 65; i++) {
        recv = buffer[i];  // Read data starting from buffer[1]
        tmpbuf[i - 1] = recv;  // Store in temporary buffer
	printf("Received Data:0x%02X\n",tmpbuf[i - 1]);
        // Compare received byte with expected byte (PN9_Tab[i-1])
        if ((recv ^ PN9_Tab[i - 1]) != 0) {
            tmp = recv ^ PN9_Tab[i - 1];  // Calculate bit errors
            // Count the number of bit errors
            Err_BitCnt += (BitCount_Tab[tmp >> 4] + BitCount_Tab[tmp & 0x0F]);
            err = 1;  // Set error flag
        }
    }

    // Step 5: Check if there was a packet error
    if (err) {
        Err_Frame++;  // Increment frame error count if there was an error
    }

}
